// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>

#include <anna/app/functions.hpp>

#include <anna/comm/ClientSocket.hpp>
#include <anna/comm/Communicator.hpp>

#include "Application.hpp"
#include "Reactor.hpp"
#include "Context.hpp"
#include "Transaction.hpp"

using namespace std;
using namespace anna;
using namespace test;

/**
 * Recupera la respuesta del servidor que usa ANNA.comm y lo transfiere al
 * cliente original que usaba el protocolo HTTP.
 */
void http4comm::Reactor::apply (comm::ClientSocket&, const comm::Message& message) 
   throw (RuntimeException)
{
   LOGMETHOD (TraceMethod tm ("http4comm::Reactor", "apply", ANNA_FILE_LOCATION));

   comm::Communicator* communicator = app::functions::component <comm::Communicator> (ANNA_FILE_LOCATION);
   
   http4comm::Application& app = static_cast <http4comm::Application&> (app::functions::getApp ());      
   Context* context = app.getContext ();   
  
   a_response.decode (message.getBody ());

   // Como cookie entre extremos del gateway usa el inittime de la peticin
   Transaction* transaction = static_cast <http4comm::Transaction*> (context->find (a_response.initTime));

   comm::ClientSocket* clientSocket = transaction->getClientSocket ();

   if (communicator->isUsable (clientSocket) == false) {
      LOGWARNING (
         string msg (transaction->asString ());
         msg += " | Response ignoreb because of ClientSocket";
         Logger::warning (msg, ANNA_FILE_LOCATION);
      )
      return;
   }

   a_httpResponse.setStatusCode (200);
   a_httpResponse.setBody (message.getBody ());

   try {
      clientSocket->send (a_httpResponse);
   }
   catch (RuntimeException& ex) {
      ex.trace ();
   }

   context->close (transaction);
}

 
