// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_Message_hpp
#define anna_comm_Message_hpp

#include <anna/core/DataBlock.hpp>

namespace anna {

namespace xml {
class Compiler;
class Node;
}

namespace comm {

/**
   Clase que modela un mensaje de red basico.
*/
class Message : public DataBlock {
public:
  /**
   * Constructor
   */
  Message();

  /**
   * Destructor.
   */
  virtual ~Message();

  /**
     Devuelve la referencia al cuerpo de este mensaje.
     \return la referencia al cuerpo de este mensaje.
  */
  const DataBlock& getBody() const throw() { return *this; }

  /**
     Establece el cuerpo de este mensaje.
     \param body Bloque de memoria que contiene el mensaje.
     \return Un puntero a la direccion de este mensaje.
     \warning Si el contructor activa el sistema de optimizacion de renidmiento el bloque de memoria no se
     copia, por lo que deberia estar disponible, al menos hasta que invoquemos al metodo Message::code.
  */
  Message * setBody(const DataBlock& body) throw(RuntimeException) { assign(body); return this; }

  /**
     Establece el cuerpo de este mensaje.
     \param buffer Bloque de memoria que contiene el mensaje.
     \param size Numero de bytes que contiene el mensaje.
     \return Un puntero a la direccion de este mensaje.
     \warning El bloque de memoria no se copia, por lo que deberia estar disponible, al menos
     hasta que invoquemos al metodo Message::code.
  */
  Message * setBody(const char* buffer, const int size) throw(RuntimeException) {
    DataBlock aux(buffer, size, false);
    assign(aux);
    return this;
  }

  /**
   * Establece el cuerpo de este mensaje con el contenido del documento XML correspondiente al
   * nodo XML recibido como parmetro.
   * \param node Nodo XML que contiene el documento XML.
   */
  virtual Message * setBody(const xml::Node* node) throw(RuntimeException);

  /**
     Elimina el contenido del cuerpo de este mensaje.
  */
  void clearBody() throw() { clear(); }

  /**
     Codifica este mensaje, por defecto no realiza ninguna operacion y retorna
     el contenido del cuerpo tal y con este.
     \return El bloque de datos que contiene el mensaje codificado.
  */
  virtual const DataBlock& code() throw(RuntimeException) { return *this; }

protected:
  struct StatusCodeBuffer { enum _v { None, Reserve, Copy }; };

  DataBlock* a_codeBuffer;

  /**
   * constructor.
   * \param statusCodeBuffer Indica el modo de reservar la memoria adcional que pueden necesitar las clases
   * heredadas para realizar la codificacin del mensaje.
   */
  explicit Message(const StatusCodeBuffer::_v statusCodeBuffer);

  /**
   * constructor.
   * \param codeBuffer Bfer a usar como memoria adicional en caso de que la clase heredada la requiera para
   * codificar.
   */
  explicit Message(DataBlock& codeBuffer);

private:
  const StatusCodeBuffer::_v a_statusCodeBuffer;
  xml::Compiler* a_xmlCompiler;

  Message(const Message&);
  Message& operator = (const Message&);
};

}
}

#endif


