// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_mt_Guard_hpp
#define anna_core_mt_Guard_hpp

#include <anna/core/RuntimeException.hpp>
#include <anna/core/mt/Safe.hpp>

namespace anna {

/**
  Guarda de seccion critica para cualquier tipo de objeto.

  Asegura que las secciones criticas son iniciadas y terminadas correctamente, ademas
  en caso de terminar la seccion critica debido a una excepcion el mutex es liberado
  correctamente.

  De esta forma nos aseguramos que solo un unico threads sera capaz de acceder y/o modificar
  simultaneamente el valor del miembro una instancia de la clase A.

  Observar que todos los metodos que establezcan una seccion critica debe estar preparados
  para lanzar una excepcion de tipo  anna::RuntimeException.

  Para poder acceder a toda la funcionalidad de esta clase el codigo debe ser compilado
  con la definicion de la macro _MT.

  @see Safe.
*/
class Guard {
public:
  /**
    Constructor.
    @param object Sobre el que actuar para iniciar la guarda.
  */
  Guard(const Safe& object) throw(RuntimeException) : a_whatis(NULL) {
    lock(a_safeObject = const_cast <Safe*>(&object), NULL);
  }

  /**
    Constructor.
    @param object Sobre el que actuar para iniciar la guarda.
  */
  Guard(const Safe* object) throw(RuntimeException);

  /**
    Constructor.
    @param object Sobre el que actuar para iniciar la guarda.
    @param whatis Texto que indica la naturaleza del objeto a bloquear.
  */
  Guard(const Safe& object, const char* whatis) throw(RuntimeException) : a_whatis(whatis) {
    lock(a_safeObject = const_cast <Safe*>(&object), whatis);
  }

  /**
    Constructor.
    @param object Sobre el que actuar para iniciar la guarda.
    @param whatis Texto que indica la naturaleza del objeto a bloquear.
  */
  Guard(const Safe* object, const char* whatis) throw(RuntimeException);

  /**
    Destructor.
    Termina la guarda iniciada con el objeto recibido como parametro en el
    constructor.
  */
  virtual ~Guard() throw() { deactivate(); }

  /**
     Desactiva la seccion critica antes de que termine el ambito de actuacion de la misma.
  */
  void deactivate() throw();

private:
  Safe* a_safeObject;
  const char* a_whatis;

  void lock(Safe*, const char* whatis) throw(RuntimeException);
};

} //namespace anna

#endif
