// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_mt_Safe_hpp
#define anna_core_mt_Safe_hpp

#include <anna/core/RuntimeException.hpp>

namespace anna {

class Guard;

/**
   Clase de la que pueden heredar todas las clases seguras que vayan a usar las guardas (ver Guard).

   @see Guard
*/
class Safe {
public:
  /**
    Marca el inicio del acceso seguro a este objeto.
    \warning Cada invocacion a este metodo debe tener su correspondiente llamada al metodo #unlock.
    Es muy aconsejable delegar las llamadas a estos metodos en una instancia Guard.
  */
  virtual void lock() throw(RuntimeException) = 0;

  /**
    Indica el final del acceso seguro iniciado al invocar #lock.
    \warning Indica el final del acceso seguro iniciado al invocar #lock.
    Es muy aconsejable delegar las llamadas a estos metodos en una instancia Guard.
  */
  virtual void unlock() throw() = 0;

protected:
  /**
     Constructor.
     \param mode Modo de actuacion de esta instancia en modo ST cuando montamos una
     seccion critca sobre este objeto. En modo MT siempre sera Mode::Normal, es decir
     abre una seccion critica sobre este objeto, que bloqueara a cualquier otro thread
     que intente acceder a el.
  */
  Safe() {}
};

}

#endif
