// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_codec_tme_Avp_hpp
#define anna_diameter_codec_tme_Avp_hpp

// Local
#include <anna/diameter/codec/Avp.hpp>
#include <anna/diameter/helpers/tme/codectypes/codectypes.hpp>

//------------------------------------------------------------------------------
//---------------------------------------------------------------------- #define
//------------------------------------------------------------------------------

namespace anna {
namespace xml {
class Attribute;
}
}



namespace anna {

namespace diameter {

namespace helpers {
namespace tme {
namespace codectypes {
class Unsigned16;
class ISDNNumber;
class ISDNAddress;
}
}
}

namespace codec {

class Engine;

namespace tme {

class Avp;
class Message;
//class Engine;


using namespace helpers::tme::codectypes;

/**
* Diameter TME avp generic container
* Manages tme-specific formats (Unsigned16, ISDNAddress, ISDNNumber)
*/
class Avp : public anna::diameter::codec::Avp {

  // Data containers
  ISDNNumber *a_ISDNNumber;
  ISDNAddress *a_ISDNAddress;
  Unsigned16 *a_Unsigned16;

  virtual void initializeByFormat() throw();
  virtual U24 getLengthByFormat(const anna::diameter::stack::Format *stackFormat) const throw();
  virtual std::string getXMLdataByFormat(bool & isHex, const anna::diameter::stack::Format *stackFormat) const throw();
  virtual void fromXMLByFormat(const anna::xml::Attribute* data, const anna::xml::Attribute* hexData, const anna::diameter::stack::Format *stackFormat) throw(anna::RuntimeException);
  virtual void codeByFormat(char* dataPart, const anna::diameter::stack::Format *stackFormat) const throw(anna::RuntimeException);
  virtual void decodeDataPartByFormat(const char * buffer, int size, const anna::diameter::stack::Format *stackFormat) throw(anna::RuntimeException);
  virtual void allocationByFormat(const anna::diameter::stack::Format *stackFormat) throw();
  virtual void clearByFormat() throw();

//protected:

//  /** Codec Engine getter: avoids have to create base engine when using its child */
//  virtual anna::diameter::codec::Engine * getEngine() const throw(anna::RuntimeException);


public:

  /**
  * Default constructor
  * @param engine Codec engine used
  */
  Avp(Engine *engine = NULL) : anna::diameter::codec::Avp(engine) {;}

  /**
  * Identified constructor
  * @param id Avp identifier as pair (code,vendor-id).
  * @param engine Codec engine used
  */
  Avp(AvpId id, Engine *engine = NULL) : anna::diameter::codec::Avp(id, engine) {;}


  /**
  * Destructor
  */
  ~Avp();


  /** Access content for ISDNNumber Avp in order to set data part */
  ISDNNumber *        getISDNNumber() throw(anna::RuntimeException) { assertFormat("ISDNNumber"); return a_ISDNNumber; }
  /** Access content for ISDNAddress Avp in order to set data part */
  ISDNAddress *        getISDNAddress() throw(anna::RuntimeException) { assertFormat("ISDNAddress"); return a_ISDNAddress; }
  /** Access content for Unsigned16 Avp in order to set data part */
  Unsigned16 *        getUnsigned16() throw(anna::RuntimeException) { assertFormat("Unsigned16"); return a_Unsigned16; }


  /** Access content for ISDNNumber Avp */
  const ISDNNumber *         getISDNNumber() const throw(anna::RuntimeException) { assertFormat("ISDNNumber"); return a_ISDNNumber; }
  /** Access content for ISDNAddress Avp */
  const ISDNAddress *        getISDNAddress() const throw(anna::RuntimeException) { assertFormat("ISDNAddress"); return a_ISDNAddress; }
  /** Access content for Unsigned16 Avp */
  const Unsigned16 *        getUnsigned16() const throw(anna::RuntimeException) { assertFormat("Unsigned16"); return a_Unsigned16; }




  friend class Message;
  friend class anna::diameter::codec::Engine;
};

}
}
}
}


#endif
