// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <algorithm>

#include <anna/core/tracing/Logger.hpp>

#include <anna/app/Component.hpp>
#include <anna/app/functions.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

using namespace std;
using namespace anna;

app::Component::Component(const char* className) :
  anna::Component(className),
  a_state(State::Stopped) {
  Application& app(app::functions::getApp());

  if(app.a_running == false) {
    try {
      app.attach(this);
    } catch(RuntimeException& ex) {
      ex.trace();
    }
  } else if(Logger::isActive(Logger::Warning) == true) {
    string msg(asString());
    msg += " | Application already running. You may initialize this component manually (lazy initialization)";
    Logger::warning(msg, ANNA_FILE_LOCATION);
  }
}

app::Component::~Component() {
  try {
    functions::getApp().detach(this);
  } catch(RuntimeException& ex) {
    ex.trace();
  }
}

void app::Component::addPredecessor(const char* componentName)
throw() {
  const std::string name(componentName);

  if((find(begin(), end(), name)) != end())
    return;

  a_predecessors.push_back(name);
  LOGDEBUG(
    string msg("anna::app::Component::addPredecessor | ");
    msg += asString();
    msg += " | Requires: ";
    msg += name;
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
}

void app::Component::initialize()
throw(RuntimeException) {
  if(a_state == State::Running)
    return;

  a_state = State::Starting;
  Application& app = functions::getApp();
  Component* predecessor;

  try {
    for(iterator ii = begin(), maxii = end(); ii != maxii; ii ++) {
      const std::string& name = data(ii);

      if((predecessor = app.find(name.c_str())) == NULL) {
        string msg("anna::app::Component::initialize | ");
        msg += asString();
        msg += " | Requires component '";
        msg += name;
        msg += "' undefined";
        throw RuntimeException(msg, ANNA_FILE_LOCATION);
      }

      if(predecessor->a_state == State::Starting) {
        string msg("anna::app::Component::initialize | ");
        msg += asString();
        msg += " | Cyclic dependency with '";
        msg += name;
        msg += "'";
        throw RuntimeException(msg, ANNA_FILE_LOCATION);
      }

      predecessor->initialize();
    }

    do_initialize();
    a_state = State::Running;
  } catch(RuntimeException&) {
    a_state = State::Stopped;
    throw;
  }
}

void app::Component::attach()
throw(RuntimeException) {
  app::functions::getApp().attach(this);
}

std::string app::Component::asString() const
throw() {
  return anna::Component::asString();
}

xml::Node* app::Component::asXML(xml::Node* parent) const
throw() {
  return anna::Component::asXML(parent);
}

