// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceWriter.hpp>
#include <anna/core/functions.hpp>
#include <anna/xml/Node.hpp>

// Local
#include <anna/core/oam/Configuration.hpp>


//******************************************************************************
//---------------------------------------------------------------- Configuration
//******************************************************************************


//------------------------------------------------------------------------------
//----------------------------------------------- Configuration::Configuration()
//------------------------------------------------------------------------------
anna::oam::Configuration::Configuration() {
  //a_alarm_text_preffix_components.clear();
  //a_alarm_text_suffix_components.clear();
  a_alarms_preffix_enabled = true;
  a_alarms_suffix_enabled = true;
  // Alarm Text Delimiters:
  a_alarm_text_delimiter_zS = ' ';
  a_alarm_text_delimiter_psL = "[ ";
  a_alarm_text_delimiter_psS = " | ";
  a_alarm_text_delimiter_psR = " ]";
};


//------------------------------------------------------------------------------
//-------------------------------------- Configuration::setAlarmTextDelimiters()
//------------------------------------------------------------------------------
void anna::oam::Configuration::setAlarmTextDelimiters(const char zS, const std::string & psL, const std::string & psS, const std::string & psR) throw() {
  a_alarm_text_delimiter_zS = zS;
  a_alarm_text_delimiter_psL = psL;
  a_alarm_text_delimiter_psS = psS;
  a_alarm_text_delimiter_psR = psR;
}


//------------------------------------------------------------------------------
//-------------------------------------- Configuration::getAlarmTextDelimiters()
//------------------------------------------------------------------------------
void anna::oam::Configuration::getAlarmTextDelimiters(char & zS, std::string & psL, std::string & psS, std::string & psR) const throw() {
  zS = a_alarm_text_delimiter_zS;
  psL = a_alarm_text_delimiter_psL;
  psS = a_alarm_text_delimiter_psS;
  psR = a_alarm_text_delimiter_psR;
}


//------------------------------------------------------------------------------
//----------------------------------------- Configuration::enableAlarmsPreffix()
//------------------------------------------------------------------------------
void anna::oam::Configuration::enableAlarmsPreffix(void) throw() {
  a_alarms_preffix_enabled = true;
  LOGDEBUG(anna::Logger::debug("Alarm preffix global components SHOWN", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//------------------------------------------ Configuration::enableAlarmsSuffix()
//------------------------------------------------------------------------------
void anna::oam::Configuration::enableAlarmsSuffix(void) throw() {
  a_alarms_suffix_enabled = true;
  LOGDEBUG(anna::Logger::debug("Alarm suffix global components SHOWN", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//---------------------------------------- Configuration::disableAlarmsPreffix()
//------------------------------------------------------------------------------
void anna::oam::Configuration::disableAlarmsPreffix(void) throw() {
  a_alarms_preffix_enabled = false;
  LOGDEBUG(anna::Logger::debug("Alarm preffix global components HIDDEN", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//----------------------------------------- Configuration::disableAlarmsSuffix()
//------------------------------------------------------------------------------
void anna::oam::Configuration::disableAlarmsSuffix(void) throw() {
  a_alarms_suffix_enabled = false;
  LOGDEBUG(anna::Logger::debug("Alarm suffix global components HIDDEN", ANNA_FILE_LOCATION));
}


//------------------------------------------------------------------------------
//------------------------------------------------------- Configuration::asXML()
//------------------------------------------------------------------------------
anna::xml::Node* anna::oam::Configuration::asXML(anna::xml::Node* parent) const throw() {
  anna::xml::Node* result = parent->createChild("anna.oam.Configuration");
  std::vector<std::string>::const_iterator it, it_min, it_max;
  result->createAttribute("AlarmsPreffix", a_alarms_preffix_enabled ? "Enabled" : "Disabled");
  it_min = a_alarm_text_preffix_components.begin();
  it_max = a_alarm_text_preffix_components.end();

  if(a_alarm_text_preffix_components.size() != 0) {
    anna::xml::Node* pc = result->createChild("AlarmsPreffixComponents");

    for(it = it_min; it != it_max; it++)
      pc->createAttribute(anna::functions::asString("Text%d", (int)(it - it_min) + 1).c_str(), *it);
  }

  result->createAttribute("AlarmsSuffix", a_alarms_suffix_enabled ? "Enabled" : "Disabled");
  it_min = a_alarm_text_suffix_components.begin();
  it_max = a_alarm_text_suffix_components.end();

  if(a_alarm_text_suffix_components.size() != 0) {
    anna::xml::Node* pc = result->createChild("AlarmsSuffixComponents");

    for(it = it_min; it != it_max; it++)
      pc->createAttribute(anna::functions::asString("Text%d", (int)(it - it_min) + 1).c_str(), *it);
  }

  char zoneDelimiterCharacter[2];
  sprintf(zoneDelimiterCharacter, "%c", a_alarm_text_delimiter_zS);
  result->createAttribute("AlarmTextZoneDelimiter", zoneDelimiterCharacter);
  result->createAttribute("AlarmPreffixSuffixLeftDelimiter", a_alarm_text_delimiter_psL);
  result->createAttribute("AlarmPreffixSuffixSeparatorDelimiter", a_alarm_text_delimiter_psS);
  result->createAttribute("AlarmPreffixSuffixRightDelimiter", a_alarm_text_delimiter_psR);
  return result;
}

