// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/core/util/MultiRangeExpression.hpp>

#include <anna/core/util/Tokenizer.hpp>
#include <anna/core/functions.hpp>

// Standard
#include <limits.h>


//using namespace anna;


// private

//------------------------------------------------------------------------------
//---------------------------------------------- MultiRangeExpression::refresh()
//------------------------------------------------------------------------------
void anna::MultiRangeExpression::refresh(void) throw() {
  anna::Tokenizer ranges, borders;
  anna::Tokenizer::const_iterator ranges_it, borders_it;
  std::string range;
  unsigned int min, max;
  a_data.clear();

  if(a_literal == "") return;

  ranges.apply(a_literal, ",");

  for(ranges_it = ranges.begin(); ranges_it != ranges.end(); ranges_it ++) {
    range = anna::Tokenizer::data(ranges_it);
    borders.apply(range, "-");
    borders_it = borders.begin();

    if(borders_it != borders.end()) {
      min = atoi(anna::Tokenizer::data(borders_it));
      max = min;
      borders_it++;

      if(borders_it != borders.end()) {
        max = atoi(anna::Tokenizer::data(borders_it));
      }

      // Update a_data:
      for(unsigned int k = min; k <= max; k++) {
        a_data[k] = 0;

        if(k == UINT_MAX/* overflow */) break;
      }
    }
  }
}


//------------------------------------------------------------------------------
//----------------------------------- MultiRangeExpression::getExpandedLiteral()
//------------------------------------------------------------------------------
std::string anna::MultiRangeExpression::getExpandedLiteral(void) const throw() {
  std::string result;
  std::map < unsigned int, int/*dummy*/ >::const_iterator it;
  std::map < unsigned int, int/*dummy*/ >::const_iterator it_min(a_data.begin());
  std::map < unsigned int, int/*dummy*/ >::const_iterator it_max(a_data.end());

  for(it = it_min; it != it_max; it++) {
    result += anna::functions::asString((*it).first);
    result += ",";
  }

  int pos = result.size();

  if(pos) result.erase(pos - 1);

  return (result);
}


//------------------------------------------------------------------------------
//-------------------------------------- MultiRangeExpression::simplifyLiteral()
//------------------------------------------------------------------------------
const char * anna::MultiRangeExpression::simplifyLiteral(void) throw() {
  if(a_data.size() == 0) return NULL;

  std::map < unsigned int, int/*dummy*/ >::const_iterator it;
  std::map < unsigned int, int/*dummy*/ >::const_iterator it_min(a_data.begin());
  std::map < unsigned int, int/*dummy*/ >::const_iterator it_max(a_data.end());
  unsigned int min = UINT_MAX;
  unsigned int max = 0;
  unsigned int value;
  unsigned int prevValue = a_data.begin()->first;
  a_literal = "";

  for(it = it_min; it != it_max; it++) {
    value = (*it).first;

    if(value < min) min = value;

    if(value - prevValue > 1) {
      a_literal += anna::functions::asString(min);
      a_literal += "-";
      a_literal += anna::functions::asString(max);
      a_literal += ",";
      min = value;
    }

    if(value > max) max = value;

    prevValue = value;
  }

  a_literal += anna::functions::asString(min);

  if(max != min) {
    a_literal += "-";
    a_literal += anna::functions::asString(max);
  }

  return a_literal.c_str();
}

