// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/tracing/Logger.hpp>

#include <anna/app/functions.hpp>

#include <anna/timex/Engine.hpp>

#include <anna/ldap/TimerManager.hpp>
#include <anna/ldap/internal/Timer.hpp>
#include <anna/ldap/Response.hpp>
#include <anna/ldap/Session.hpp>

using namespace std;
using namespace anna;

ldap::TimerManager::TimerManager() :
  timex::TimeEventObserver("anna::ldap::TimerManager"),
  a_timeController(NULL) {
}

//-------------------------------------------------------------------------------------------------------
// (1) Bloquea el TimerManager el primero para mantener siempre el mismo orden de acceso a la
// seccion critica, lo que evita interbloqueos.
//-------------------------------------------------------------------------------------------------------
ldap::Timer* ldap::TimerManager::createTimer(Response* response)
throw(RuntimeException) {
  Timer* result(NULL);

  if(a_timeController == NULL)
    a_timeController = app::functions::component <timex::Engine> (ANNA_FILE_LOCATION);

  Guard guard(a_timeController, "anna::ldap::TimerManager::createTimer");              // (1)
  result = a_timers.create();
  const ClassCode::_v v = response->getClassCode();
  result->setId((timex::TimeEvent::Id) response);
  result->setObserver(this);
  result->setResponse(response);
  result->setTimeout(response->getSession()->getTimeout(v));
  LOGDEBUG(
    string msg("anna::ldap::TimerManager::createTimer | ");
    msg += result->asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );
  a_timeController->activate(result);
  return result;
}

void ldap::TimerManager::cancel(ldap::Timer* timer)
throw() {
  if(timer == NULL)
    return;

  LOGDEBUG(
    string msg("anna::ldap::TimerManager::cancel | ");
    msg += timer->asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );

  try {
    if(a_timeController == NULL)
      a_timeController = app::functions::component <timex::Engine> (ANNA_FILE_LOCATION);

    a_timeController->cancel(timer);
  } catch(RuntimeException& ex) {
    ex.trace();
  }
}

//------------------------------------------------------------------------------------------
// Se invoca automaticamente desde timex::Engine
//------------------------------------------------------------------------------------------
void ldap::TimerManager::release(timex::TimeEvent* timeEvent)
throw() {
  Timer* timer = static_cast <Timer*>(timeEvent);
  timer->setResponse(NULL);
  a_timers.release(timer);
}

