// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <unistd.h>
#include <signal.h>

#include <anna/core/functions.hpp>
#include <anna/core/tracing/Logger.hpp>

#include <anna/app/functions.hpp>
#include <anna/app/Application.hpp>

#include <anna/comm/Communicator.hpp>

#include <anna/timex/internal/TickProducer.hpp>
#include <anna/timex/Engine.hpp>

using namespace std;
using namespace anna;

//static
void* anna::timex::TickProducer::exec(void* arg)
throw() {
  sigset_t mask;
  sigemptyset(&mask);
  pthread_sigmask(SIG_SETMASK, &mask, NULL);
  timex::TickProducer* _this = reinterpret_cast <timex::TickProducer*>(arg);

  try {
    comm::Communicator* communicator = app::functions::component <comm::Communicator> (ANNA_FILE_LOCATION);

    while(communicator->isServing() == false)
      functions::sleep(Millisecond(100));

    while(_this->a_requestedStop == false) {
      functions::sleep(_this->calculeSlice(functions::millisecond()));
      _this->tick();
    }
  } catch(RuntimeException& ex) {
    Logger::write(Logger::Critical, ex.getText(), ANNA_FILE_LOCATION);
    exit(-1);
  }

  return NULL;
}

anna::timex::TickProducer::TickProducer(timex::Engine* timeController, const int fdWrite) :
  a_timeController(*timeController),
  a_fdWrite(fdWrite),
  a_expectedTime(0),
  a_isInPause(false),
  a_requestedStop(false)
{;}

void anna::timex::TickProducer::tick()
throw() {
  static char onebyte = 0xff;
  static int errorCount = 0;

  if(a_isInPause == true)
    return;

  if(write(a_fdWrite, &onebyte, sizeof(onebyte)) < 0) {
    Logger::write(Logger::Critical, "Cannot generate tick", strerror(errno), ANNA_FILE_LOCATION);

    if(++ errorCount == 10) {
      Logger::write(Logger::Critical, "Terminating application due to error at tick generation", ANNA_FILE_LOCATION);
      app::functions::getApp().eventAbnormalTermination("timex::TickProducer");
      _exit(-1);
    }
  }
}

//--------------------------------------------------------------------------------------------
// Calcula la duracion del proximo tick de reloj. Tiene en cuenta la deriva del anterior
// tick y la corrige en este.
// (2) Si es mayor que cero => El reloj del sistema se adelanta
//     Si es menor que cero => El reloj del sistema se atrasa.
// (3) Si la correcin aplicada es mayor del 5% de la resolucin volvemos a calcularla ya
//     que podemos considerar que normalmente no desviar�tanto tiempo.
//--------------------------------------------------------------------------------------------
Millisecond anna::timex::TickProducer::calculeSlice(const Millisecond & msnow)
throw() {
  Millisecond result(a_timeController.getResolution());
  int correction = (a_expectedTime == 0) ? 0 : (a_expectedTime - msnow);           // (2)

  if(correction != 0) {
    if(((abs(correction) * 100) / result) > 5)                                       // (3)
      correction = 0;

    result += Millisecond(correction);
  }

  a_expectedTime = msnow + result;
  return result;
}


